﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch.Knapsack {
  /// <summary>
  /// An operator that performs diversity calculation between bool-valued vectors.
  /// </summary>
  [Item("BinaryVectorDiversityCalculator", "An operator that performs diversity calculation between bool-valued vectors.")]
  [StorableClass]
  public sealed class BinaryVectorDiversityCalculator : DiversityCalculator {
    [StorableConstructor]
    private BinaryVectorDiversityCalculator(bool deserializing) : base(deserializing) { }
    private BinaryVectorDiversityCalculator(BinaryVectorDiversityCalculator original, Cloner cloner) : base(original, cloner) { }
    public BinaryVectorDiversityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new BinaryVectorDiversityCalculator(this, cloner);
    }

    protected override double CalculateDiversity(IItem left, IItem right) {
      if (!(left is BinaryVector) || !(right is BinaryVector))
        throw new ArgumentException("Cannot calculate diversity because one of the provided objects or both have the wrong type.");

      var v1 = left as BinaryVector;
      var v2 = right as BinaryVector;
      double diversity = 0;
      for (int i = 0; i < v1.Length; i++)
        if (v1[i] != v2[i]) diversity++;
      return diversity;
    }
  }
}
