﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch.TestFunctions {
  /// <summary>
  /// An operator that performs diversity calculation between real vectors.
  /// </summary>
  [Item("RealVectorDiversityCalculator", "An operator that performs diversity calculation between real vectors.")]
  [StorableClass]
  public sealed class RealVectorDiversityCalculator : DiversityCalculator {
    [StorableConstructor]
    private RealVectorDiversityCalculator(bool deserializing) : base(deserializing) { }
    private RealVectorDiversityCalculator(RealVectorDiversityCalculator original, Cloner cloner) : base(original, cloner) { }
    public RealVectorDiversityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RealVectorDiversityCalculator(this, cloner);
    }

    protected override double CalculateDiversity(IItem left, IItem right) {
      if (!(left is RealVector) || !(right is RealVector))
        throw new ArgumentException("Cannot calculate diversity because one of the provided objects or both have the wrong type.");

      var v1 = left as RealVector;
      var v2 = right as RealVector;
      double sum = 0.0;
      for (int i = 0; i < v1.Length; i++)
        sum += Math.Pow(v1[i] - v2[i], 2);
      return Math.Sqrt(sum);
    }
  }
}
