﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.TestFunctions;

namespace HeuristicLab.Algorithms.ScatterSearch.TestFunctions {
  /// <summary>
  /// An operator that improves Rosenbrock test function solutions.
  /// </summary>
  [Item("TestFunctionsRosenbrockImprovementOperator", "An operator that improves Rosenbrock test function solutions.")]
  [StorableClass]
  public sealed class TestFunctionsRosenbrockImprovementOperator : TestFunctionsImprovementOperator {
    [StorableConstructor]
    private TestFunctionsRosenbrockImprovementOperator(bool deserializing) : base(deserializing) { }
    private TestFunctionsRosenbrockImprovementOperator(TestFunctionsRosenbrockImprovementOperator original, Cloner cloner) : base(original, cloner) { }
    public TestFunctionsRosenbrockImprovementOperator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TestFunctionsRosenbrockImprovementOperator(this, cloner);
    }

    public override IOperation Apply() {
      FunctionEvaluator = x => { return RosenbrockEvaluator.Apply(x); };
      return base.Apply();
    }
  }
}
